@@ -1,4 +1,4 @@
-# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2003-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This file is part of logilab-common.
@@ -15,12 +15,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public License along
 # with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
-"""Date manipulation helper functions.
+"""Date manipulation helper functions."""
 
-
-
-
-"""
 __docformat__ = "restructuredtext en"
 
 import math
@@ -33,8 +29,6 @@ from calendar import monthrange, timegm
 try:
     from mx.DateTime import RelativeDateTime, Date, DateTimeType
 except ImportError:
-    from warnings import warn
-    warn("mxDateTime not found, endOfMonth won't be available")
     endOfMonth = None
     DateTimeType = datetime
 else:
@@ -44,52 +38,52 @@ else:
 #       as we have in lgc.db ?
 
 FRENCH_FIXED_HOLIDAYS = {
-    'jour_an'        : '%s-01-01',
-    'fete_travail'   : '%s-05-01',
-    'armistice1945'  : '%s-05-08',
-    'fete_nat'       : '%s-07-14',
-    'assomption'     : '%s-08-15',
-    'toussaint'      : '%s-11-01',
-    'armistice1918'  : '%s-11-11',
-    'noel'           : '%s-12-25',
+    'jour_an': '%s-01-01',
+    'fete_travail': '%s-05-01',
+    'armistice1945': '%s-05-08',
+    'fete_nat': '%s-07-14',
+    'assomption': '%s-08-15',
+    'toussaint': '%s-11-01',
+    'armistice1918': '%s-11-11',
+    'noel': '%s-12-25',
     }
 
 FRENCH_MOBILE_HOLIDAYS = {
-    'paques2004'    : '2004-04-12',
-    'ascension2004' : '2004-05-20',
-    'pentecote2004' : '2004-05-31',
+    'paques2004': '2004-04-12',
+    'ascension2004': '2004-05-20',
+    'pentecote2004': '2004-05-31',
 
-    'paques2005'    : '2005-03-28',
-    'ascension2005' : '2005-05-05',
-    'pentecote2005' : '2005-05-16',
+    'paques2005': '2005-03-28',
+    'ascension2005': '2005-05-05',
+    'pentecote2005': '2005-05-16',
 
-    'paques2006'    : '2006-04-17',
-    'ascension2006' : '2006-05-25',
-    'pentecote2006' : '2006-06-05',
+    'paques2006': '2006-04-17',
+    'ascension2006': '2006-05-25',
+    'pentecote2006': '2006-06-05',
 
-    'paques2007'    : '2007-04-09',
-    'ascension2007' : '2007-05-17',
-    'pentecote2007' : '2007-05-28',
+    'paques2007': '2007-04-09',
+    'ascension2007': '2007-05-17',
+    'pentecote2007': '2007-05-28',
 
-    'paques2008'    : '2008-03-24',
-    'ascension2008' : '2008-05-01',
-    'pentecote2008' : '2008-05-12',
+    'paques2008': '2008-03-24',
+    'ascension2008': '2008-05-01',
+    'pentecote2008': '2008-05-12',
 
-    'paques2009'    : '2009-04-13',
-    'ascension2009' : '2009-05-21',
-    'pentecote2009' : '2009-06-01',
+    'paques2009': '2009-04-13',
+    'ascension2009': '2009-05-21',
+    'pentecote2009': '2009-06-01',
 
-    'paques2010'    : '2010-04-05',
-    'ascension2010' : '2010-05-13',
-    'pentecote2010' : '2010-05-24',
+    'paques2010': '2010-04-05',
+    'ascension2010': '2010-05-13',
+    'pentecote2010': '2010-05-24',
 
-    'paques2011'    : '2011-04-25',
-    'ascension2011' : '2011-06-02',
-    'pentecote2011' : '2011-06-13',
+    'paques2011': '2011-04-25',
+    'ascension2011': '2011-06-02',
+    'pentecote2011': '2011-06-13',
 
-    'paques2012'    : '2012-04-09',
-    'ascension2012' : '2012-05-17',
-    'pentecote2012' : '2012-05-28',
+    'paques2012': '2012-04-09',
+    'ascension2012': '2012-05-17',
+    'pentecote2012': '2012-05-28',
     }
 
 # XXX this implementation cries for multimethod dispatching
@@ -241,6 +235,19 @@ def todatetime(somedate):
 def datetime2ticks(somedate):
     return timegm(somedate.timetuple()) * 1000
 
+def ticks2datetime(ticks):
+    miliseconds, microseconds = divmod(ticks, 1000)
+    try:
+        return datetime.fromtimestamp(miliseconds)
+    except (ValueError, OverflowError):
+        epoch = datetime.fromtimestamp(0)
+        nb_days, seconds = divmod(int(miliseconds), 86400)
+        delta = timedelta(nb_days, seconds=seconds, microseconds=microseconds)
+        try:
+            return epoch + delta
+        except (ValueError, OverflowError):
+            raise
+
 def days_in_month(somedate):
     return monthrange(somedate.year, somedate.month)[1]
 
@@ -279,7 +286,7 @@ def ustrftime(somedate, fmt='%Y-%m-%d'):
     try:
         return unicode(somedate.strftime(str(fmt)), encoding)
     except ValueError, exc:
-        if '1900' not in exc.args[0]:
+        if somedate.year >= 1900:
             raise
         # datetime is not happy with dates before 1900
         # we try to work around this, assuming a simple
@@ -294,3 +301,9 @@ def ustrftime(somedate, fmt='%Y-%m-%d'):
                            'S': somedate.second})
         fmt = re.sub('%([YmdHMS])', r'%(\1)02d', fmt)
         return unicode(fmt) % fields
+
+def utcdatetime(dt):
+    return datetime(*dt.utctimetuple()[:7])
+
+def utctime(dt):
+    return (dt + dt.utcoffset() + dt.dst()).replace(tzinfo=None)
